<?php

declare(strict_types=1);

namespace Drupal\media_contextual_crop_embed\Plugin\CKEditor5Plugin;

use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableInterface;
use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableTrait;
use Drupal\ckeditor5\Plugin\CKEditor5PluginDefault;
use Drupal\ckeditor5\Plugin\CKEditor5PluginElementsSubsetInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\editor\EditorInterface;
use Drupal\media_contextual_crop_embed\MediaContextualCropEmbedService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * CKEditor 5 Advanced Link plugin.
 */
class ContextualCrop extends CKEditor5PluginDefault implements ContainerFactoryPluginInterface, CKEditor5PluginConfigurableInterface, CKEditor5PluginElementsSubsetInterface {

  use CKEditor5PluginConfigurableTrait;

  /**
   * The current route service.
   *
   * @var \Drupal\media_contextual_crop_embed\MediaContextualCropEmbedService
   */
  protected $mediaContextualCropEmbedService;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a MultiCropEmbed object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity Type manager.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param \Drupal\media_contextual_crop_embed\MediaContextualCropEmbedService $media_contextual_crop_embed_service
   *   The media_contextual_crop_embed service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EntityDisplayRepositoryInterface $entity_display_repository, MediaContextualCropEmbedService $media_contextual_crop_embed_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->mediaContextualCropEmbedService = $media_contextual_crop_embed_service;
    $this->entityDisplayRepository = $entity_display_repository;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository'),
      $container->get('media_contextual_crop_embed.service')
    );
  }

  /**
 * The default configuration for this plugin.
 *
 * @var string[][]
 */
  const DEFAULT_CONFIGURATION = [
    'view_mode' => 'default',
  ];

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return static::DEFAULT_CONFIGURATION;
  }

  /**
   * Gets all enabled attributes.
   *
   * @return string[]
   *   The values in the plugins.ckeditor5_link.enabled_attributes config.
   */
  private function getEnabledAttributes(): array {
    return ['view_mode' => $this->configuration['view_mode']];
  }

  /**
   * {@inheritdoc}
   *
   * Form for choosing which additional <a> attributes are available.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $view_mode_options = $this->entityDisplayRepository->getViewModeOptions('media');

    $form['view_mode'] = [
      '#type' => 'select',
      '#options' => $view_mode_options,
      '#title' => $this->t('View mode used in Crop modal'),
      '#default_value' => $this->configuration['view_mode'],
      '#description' => $this->t('Please choose a view mode for which the image style uses a crop.'),
    ];

    // Check if media embed plugin allow_view_mode_override.
    $storage = $form_state->getStorage();
    $editor_settings = $storage['editor']->getSettings();
    $media_embed_settings = $editor_settings['plugins']['media_media']['allow_view_mode_override'];
    if ($media_embed_settings === TRUE) {

      $form['info'] = [
        '#markup' => $this->t("Media embed allows the user to choose which view mode to use. This view mode may not use a CROP."),
      ];

    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['view_mode'] = $form_state->getValue('view_mode');
  }

  /**
   * {@inheritdoc}
   *
   * Filters the enabled attributes to those chosen in editor config.
   */
  public function getDynamicPluginConfig(array $static_plugin_config, EditorInterface $editor): array {
    return [
      'MCCE' => $this->getEnabledAttributes(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getElementsSubset(): array {
    $subset[] = '<drupal-media>';
    $subset[] = '<drupal-media data-crop-settings data-crop-type>';
    return $subset;
  }

}
